@@ -1,5 +1,5 @@
 ## File Name: normal2.cw.R
-## File Version: 0.14
+## File Version: 0.16
 
 
 #** approximation of the bivariate normal integral
@@ -29,7 +29,7 @@ normal2.cw <- function( a, b, rho )
         a1[ind] <- t1[ind]
     }
     t1 <- stats::pnorm(-a1)
-    mu <- stats::dnorm(a1) / stats::pnorm(-a1)
+    mu <- sirt_dnorm(a1) / stats::pnorm(-a1)
     xi <- ( rho * mu - b1 ) / sqrt( 1 - rho^2 )
     # see Hong (1999)
     # sig2 <- 1 + mu - mu^2 #=> formula in Cox & Wermuth (1991)
@@ -37,8 +37,7 @@ normal2.cw <- function( a, b, rho )
     #  Formula (3)
     # t1 * pnorm( xi )
     # Formula (4): correction in Hong (1999)
-    prob1 <- t1 * ( stats::pnorm(  xi ) - 1/2 * rho^2 / ( 1 - rho^2 ) * xi *
-                stats::dnorm( xi ) * sig2  )
+    prob1 <- t1*( stats::pnorm(xi) - 1/2 * rho^2/( 1 - rho^2 )*xi*sirt_dnorm(xi)*sig2  )
     # adjust formula in case of APPROX. (ii)
     if ( length(ind2) > 0 ){
         # CW. Formula in (ii), p. 264
