@@ -1,18 +1,17 @@
 ## File Name: rasch.copula.R
-## File Version: 0.999
+## File Version: 0.99907
 
 
 
 #------------------------------------------------------------------
 # Copula estimation in a raschtype model
-##NS export(rasch.copula)
-rasch.copula <- function( dat, itemcluster, group = NULL,
-                        progress = TRUE, mmliter = 1000, delta = NULL,
-                        theta.k = seq(-4,4,len=21), alpha1=0, alpha2=0, numdiff.parm = .000001,
-                        est.b = seq(1,ncol(dat)), est.a = rep(1,ncol(dat)), est.delta = NULL,
-                        est.alpha = FALSE,
-                        glob.conv = .001, alpha.conv = .0001, conv1 = .001
-                                        ){
+rasch.copula <- function( dat, itemcluster, group=NULL,
+                        progress=TRUE, mmliter=1000, delta=NULL,
+                        theta.k=seq(-4,4,len=21), alpha1=0, alpha2=0, numdiff.parm=.000001,
+                        est.b=seq(1,ncol(dat)), est.a=rep(1,ncol(dat)), est.delta=NULL,
+                        est.alpha=FALSE,
+                        glob.conv=.001, alpha.conv=.0001, conv1=.001 )
+{
     ###############################################################
     # INPUT:
     # dat    ... data frame
@@ -29,8 +28,8 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
     # item clusters
     t1 <- table(itemcluster)
     t1b <- as.numeric( names(t1))
-    t1c <- t1[ names(t1) != 0 ]
-    if ( any( t1c == 1) ){
+    t1c <- t1[ names(t1) !=0 ]
+    if ( any( t1c==1) ){
            stop( "There should be at least two items in an item cluster\n" )
                                     }
             x1 <- seq( 1, max(t1b) )
@@ -71,8 +70,8 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
     if (G > 1 ){
         for (gg in 1:G){
         # gg <- 1
-        t1 <- table( patt[ group == gg  ] )
-        pattern <- merge( pattern, t1, by.x=1, by.y = 1, all=T )
+        t1 <- table( patt[ group==gg  ] )
+        pattern <- merge( pattern, t1, by.x=1, by.y=1, all=T )
                     }
         pattern[ is.na(pattern) ] <- 0
         colnames(pattern)[-c(1:2)]     <- paste("freqwgt", 1:G, sep="")
@@ -81,7 +80,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
     for (ii in 1:I){
             dat0[,ii] <- as.numeric( substring( paste( pattern[,1] ), ii+1, ii+1 ) )
                 }
-    dat0[ dat0 == 9 ] <- NA
+    dat0[ dat0==9 ] <- NA
     colnames(dat0) <- colnames(dat)
     # define mu and sigma
     mu <- rep(0,G)
@@ -102,8 +101,8 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
             # item pattern
             for (cc in 1:CC){
     #            cc <- 1
-                icl.cc <- which( itemcluster == cc )
-                dp.ld.cc <- .calc.copula.itemcluster( D = length(icl.cc) )
+                icl.cc <- which( itemcluster==cc )
+                dp.ld.cc <- .calc.copula.itemcluster( D=length(icl.cc) )
                 dp.ld.cc$items <- icl.cc
                 dp.ld.cc$N.items <- NCC <- length(icl.cc)
                 dp.ld.cc$itemlabels <- colnames(dat)[icl.cc]
@@ -111,7 +110,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 m1 <- outer( rep(1,2^NCC), icl.cc)
                 m2 <- dp.ld.cc$patt * m1   + ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                 m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
-                res <- list( "items" = m2 )
+                res <- list( "items"=m2 )
                 res1 <- rep(NCC, 2^NCC )
                 names(res1) <- rownames( dp.ld.cc$patt )
                 res$N.Index1 <- res1
@@ -120,8 +119,8 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 m2 <- ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                 m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                 m2 <- c( m2[ m2> 0 ], 2*I + 1 )
-                res <- list( "items" = m2 )
-                res$N.Index1 <- rowSums( dp.ld.cc$patt  == 0 )
+                res <- list( "items"=m2 )
+                res$N.Index1 <- rowSums( dp.ld.cc$patt==0 )
                 res$N.Index1[ length(res$N.Index1) ] <- 1
                 dp.ld.cc$dependent <- res
                 dp.ld[[cc]] <- dp.ld.cc
@@ -140,7 +139,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 dat2.ld[, cc ] <- match( l1, rownames( dp.cc$patt ) )
                 dat2.ld[ rowSums( dat2.resp[, dp.cc$items ]    ) < length(dp.cc$items), cc ] <- NA
                 NRR <- nrow( dp.cc$patt )
-                dat3.ld.cc <- sapply( seq( 1, NRR), FUN = function(aa){ 1*(dat2.ld[,cc] == aa ) } )
+                dat3.ld.cc <- sapply( seq( 1, NRR), FUN=function(aa){ 1*(dat2.ld[,cc]==aa ) } )
                 dat3.ld.cc[ is.na(dat3.ld.cc) ] <- 0
                 dat3.ld[[cc]] <- dat3.ld.cc
                             }
@@ -149,7 +148,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 # set missings in dat2 to some arbitrary category
                 dat2.ld[ is.na( dat2.ld ) ] <- 1
                 # create dat2 data sets for local independence items
-                itemcluster0 <- ind2 <- which( itemcluster == 0 )
+                itemcluster0 <- ind2 <- which( itemcluster==0 )
                 bdat2.li.resp <- dat2.li.resp <- bdat2.li <- dat2.li <- NULL
 
                 if ( length(ind2) > 0 ){
@@ -161,7 +160,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         #########################################################################
     #--------------------------------------------------
     # initial estimate of item difficulty
-#    b <- rasch.pairwise( dat, progress = FALSE)$item$itemdiff
+#    b <- rasch.pairwise( dat, progress=FALSE)$item$itemdiff
     b <- - stats::qlogis( ( colMeans( dat00, na.rm=T ) + .005 ) / 1.01 )
     I <- ncol(dat2)
     # initial estimate of (mean) item discrimination
@@ -172,7 +171,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
     if ( G > 1){
         wgt.theta <- matrix(0, length(theta.k), G )
         for ( gg in 1:G){
-            wgt.theta[,gg] <- stats::dnorm( theta.k, mean = mu[gg], sd = sigma[gg] )
+            wgt.theta[,gg] <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
             wgt.theta[,gg] <- wgt.theta[,gg] / sum( wgt.theta[,gg] )
                         }
                 }
@@ -200,13 +199,13 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         b0 <- b
         # identify different b parameter groups
         bG <- setdiff( unique( est.b ), 0 )
-        prbar <- seq( 1, 10, len = length(bG) )
+        prbar <- seq( 1, 10, len=length(bG) )
         prbar <- floor( prbar )
         prbar <- c( prbar[1], diff(prbar) )
         cat(" Estimation of b:     |")
         for (bb in bG){
-            est.bb <- 1 * (est.b == bb )
-            if (bb == 1 ){
+            est.bb <- 1 * (est.b==bb )
+            if (bb==1 ){
                 rescop <- .ll.rasch.copula2( theta.k, b0, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                             bdat2.li, bdat2.li.resp, pattern, GG  )
@@ -221,7 +220,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
 
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
             b.change <- - d1 / d2
@@ -239,12 +238,12 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         a0 <- a
         # identify different a parameter groups
         aG <- setdiff( unique( est.a ), 0 )
-        prbar <- seq( 1, 10, len = length(aG) )
+        prbar <- seq( 1, 10, len=length(aG) )
         prbar <- floor( prbar )
         prbar <- c( prbar[1], diff(prbar) )
         cat(" Estimation of a:     |")
         for (aa in aG){
-            est.aa <- 1 * (est.a == aa )
+            est.aa <- 1 * (est.a==aa )
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
@@ -257,7 +256,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             a.change <- - d1 / d2
             a.change <- ifelse( abs( a.change ) > .3, .3*sign(a.change), a.change )
@@ -275,13 +274,13 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         # estimation of delta parameters
         delta0 <- delta
         dG <- setdiff( unique( est.delta ), 0 )
-        prbar <- seq( 1, 10, len = length(dG) )
+        prbar <- seq( 1, 10, len=length(dG) )
         prbar <- floor( prbar )
         prbar <- c( prbar[1], diff(prbar) )
         # identify different a parameter groups
         cat(" Estimation of delta: |")
         for (cc in dG){
-            est.cc <- 1 * ( est.delta == cc )
+            est.cc <- 1 * ( est.delta==cc )
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                             delta, wgt.theta, I,
@@ -295,16 +294,16 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             delta.change <- - d1 / d2
             delta.change <- ifelse( abs( delta.change ) > .2, .2*sign(delta.change), delta.change )
             delta.change <- delta.change * est.cc
             delta <- delta + delta.change
-            delta[ delta <= 0 ] <- 2*numdiff.parm
-            delta[ delta >= 1 ] <- 1 - 2*numdiff.parm
+            delta[ delta <=0 ] <- 2*numdiff.parm
+            delta[ delta >=1 ] <- 1 - 2*numdiff.parm
 # adapt delta boundary estimate
-#if ( adapt.delta.boundary & ( iter %% 5 == 0 ) ){
+#if ( adapt.delta.boundary & ( iter %% 5==0 ) ){
 # print("TRUE")
 #    delta <- ifelse ( abs( delta - 2*numdiff.parm ) < 10^(-15), .20, delta )
 #                }
@@ -335,7 +334,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             alpha.change <- - d1 / d2
             a1k1 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
@@ -357,7 +356,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG)$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             alpha.change <- - d1 / d2
             a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
@@ -375,48 +374,48 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         # identify different a parameter groups
 #        h <- 10 * numdiff.parm
 #        dG <- setdiff( unique( est.delta ), 0 )
-#        prbar <- seq( 1, 10, len = length(dG) )
+#        prbar <- seq( 1, 10, len=length(dG) )
 #        prbar <- floor( prbar )
 #        prbar <- c( prbar[1], diff(prbar) )
 #        muG <- GG-1
         muG <- 1:(GG-1)
-        prbar <- seq( 1, 10, len = length(muG) )
+        prbar <- seq( 1, 10, len=length(muG) )
         prbar <- floor( prbar )
         prbar <- c( prbar[1], diff(prbar) )
         cat(" Estimation of mu:    |")
         for (gg in 2:G){
-#            est.aa <- est.a * (est.a == aa )
+#            est.aa <- est.a * (est.a==aa )
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll0 <- rescop$ll
             # mu + h
             w1 <- wgt.theta
-            w2 <- stats::dnorm( theta.k, mean = mu[gg] + h, sd = sigma[gg] )
+            w2 <- stats::dnorm( theta.k, mean=mu[gg] + h, sd=sigma[gg] )
             w1[,gg] <- w2 / sum(w2)
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
-                            wgt.theta = w1, I,
+                            wgt.theta=w1, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll1 <- rescop$ll
             # mu - h
             w1 <- wgt.theta
-            w2 <- dnorm( theta.k, mean = mu[gg] - h, sd = sigma[gg] )
+            w2 <- dnorm( theta.k, mean=mu[gg] - h, sd=sigma[gg] )
             w1[,gg] <- w2 / sum(w2)
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
-                            wgt.theta = w1, I,
+                            wgt.theta=w1, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll2 <- rescop$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             mu.change <- - d1 / d2
             mu.change <- ifelse( abs( mu.change ) > .3, .3*sign(mu.change), mu.change )
-            mu.change <- mu.change * ( ( 1:G ) == gg )
+            mu.change <- mu.change * ( ( 1:G )==gg )
             mu <- mu + mu.change
-            w2 <- stats::dnorm( theta.k, mean = mu[gg], sd = sigma[gg] )
+            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
             wgt.theta[,gg] <- w2 / sum(w2)
 #            cat( aa, " ") ;
             cat( paste( rep( "-", prbar[gg-1]), collapse="") )
@@ -435,43 +434,43 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         # identify different a parameter groups
 #        h <- 10 * numdiff.parm
         sigmaG <- seq(1,GG-1)
-        prbar <- seq( 1, 10, len = length(sigmaG) )
+        prbar <- seq( 1, 10, len=length(sigmaG) )
         prbar <- floor( prbar )
         prbar <- c( prbar[1], diff(prbar) )
         cat(" Estimation of sigma: |")
         for (gg in 2:G){
-#            est.aa <- est.a * (est.a == aa )
+#            est.aa <- est.a * (est.a==aa )
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll0 <- rescop$ll
             # sigma + h
             w1 <- wgt.theta
-            w2 <- dnorm( theta.k, mean = mu[gg], sd = sigma[gg] +h)
+            w2 <- dnorm( theta.k, mean=mu[gg], sd=sigma[gg] +h)
             w1[,gg] <- w2 / sum(w2)
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
-                            wgt.theta = w1, I,
+                            wgt.theta=w1, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll1 <- rescop$ll
             # sigma - h
             w1 <- wgt.theta
-            w2 <- stats::dnorm( theta.k, mean = mu[gg], sd = sigma[gg]-h )
+            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg]-h )
             w1[,gg] <- w2 / sum(w2)
             rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                             CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
-                            wgt.theta = w1, I,
+                            wgt.theta=w1, I,
                             bdat2.li, bdat2.li.resp, pattern, GG )
             ll2 <- rescop$ll
             d1 <- ( ll1 - ll2  ) / ( 2 * h )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
             d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             sigma.change <- - d1 / d2
             sigma.change <- ifelse( abs( sigma.change ) > .3, .3*sign(sigma.change), sigma.change )
-            sigma.change <- sigma.change * ( ( 1:G ) == gg )
+            sigma.change <- sigma.change * ( ( 1:G )==gg )
             sigma <- sigma + sigma.change
-            w2 <- stats::dnorm( theta.k, mean = mu[gg], sd = sigma[gg] )
+            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
             wgt.theta[,gg] <- w2 / sum(w2)
 #            cat( aa, " ") ;
             cat( paste( rep( "-", prbar[gg-1]), collapse="") )
@@ -488,19 +487,19 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         #**********************************************************************************
         # convergence display
         a1 <- stats::aggregate( b, list( est.b), mean )
-        cat("   b parameters: ", paste( round( a1[,2], 3 ), collapse= " " ), "\n" )
+        cat("   b parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
         a1 <- stats::aggregate( a, list( est.a), mean )
-        cat("   a parameters: ", paste( round( a1[,2], 3 ), collapse= " " ), "\n" )
-        cat("   delta parameters: ", paste( round( delta, 3 ), collapse= " " ), "\n" )
-        cat("   alpha parameters: ", paste( round( c(alpha1, alpha2), 3 ), collapse= " " ), "\n" )
-        cat("   mu parameters: ", paste( round( mu, 3 ), collapse= " " ), "\n" )
-        cat("   sigma parameters: ", paste( round( sigma, 3 ), collapse= " " ), "\n" )
+        cat("   a parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
+        cat("   delta parameters: ", paste( round( delta, 3 ), collapse=" " ), "\n" )
+        cat("   alpha parameters: ", paste( round( c(alpha1, alpha2), 3 ), collapse=" " ), "\n" )
+        cat("   mu parameters: ", paste( round( mu, 3 ), collapse=" " ), "\n" )
+        cat("   sigma parameters: ", paste( round( sigma, 3 ), collapse=" " ), "\n" )
         #******************************************************************************
         iter <- iter + 1
         dev <- -2*ll0
         dev.change <- abs( ( dev - dev0)/ dev0 )
         par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
-        cat( "Deviance = ",   round( dev, 5 ), "| max. parm. change = ",
+        cat( "Deviance=",   round( dev, 5 ), "| max. parm. change=",
                                         round( par.change, 6 ),  " \n"   )
         if ( ( dev > dev0 ) & ( iter > 4 ) ){ cat("   Deviance has increased! Convergence Problems?\n") }
         flush.console()
@@ -523,13 +522,13 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         # standard errors b
         # identify different b parameter groups
 #        bG <- setdiff( unique( est.b ), 0 )
-#        prbar <- seq( 1, 10, len = length(bG) )
+#        prbar <- seq( 1, 10, len=length(bG) )
 #        prbar <- floor( prbar )
 #        prbar <- c( prbar[1], diff(prbar) )
 #        cat(" SE's of b:            |")
 #        for (bb in bG){
-#            est.bb <- est.b * (est.b == bb )
-#            if (bb == 1 ){
+#            est.bb <- est.b * (est.b==bb )
+#            if (bb==1 ){
 #                rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
 #                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
 #                            bdat2.li, bdat2.li.resp )
@@ -545,11 +544,11 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
 #                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
 #                            bdat2.li, bdat2.li.resp, rescop, itemcluster )
             # second order derivative
-            # f(x+h)+f(x-h) = 2*f(x) + f''(x)*h^2
+            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
 #            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
 #            d2 <- sqrt( 1 / sum( - d2 ) )
-#            dfr.pp <- data.frame( "parameter" = "b", "pargropup" = bb,
-#                            "est" = b[ bb ], "se" = d2 )
+#            dfr.pp <- data.frame( "parameter"="b", "pargropup"=bb,
+#                            "est"=b[ bb ], "se"=d2 )
 #            dfr <- rbind( dfr, dfr.pp )
 #            cat( paste( rep( "-", prbar[bb]), collapse="") )
 #            flush.console()
@@ -560,7 +559,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
     #********************************************************
     # information criteria
         # calculations for information criteria
-        ic <- list( "deviance" = dev, "n" = nrow(dat00) )
+        ic <- list( "deviance"=dev, "n"=nrow(dat00) )
         # number of parameters to be estimated
         # these formulas hold when assuming normal distributions
         ic[[ "np" ]] <- length(bG) + length(aG) + length(dG) + 2*est.alpha
@@ -572,10 +571,10 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
     #**********************************************************************************
     # results item parameters
-    item <- data.frame( "item" = colnames(dat),
-                "N" = colSums(!is.na(dat00)),
-                "p" = colMeans( dat00, na.rm=T ),
-                "b" = b, "est.b"= est.b, "a" = a, "est.a" = est.a )
+    item <- data.frame( "item"=colnames(dat),
+                "N"=colSums(!is.na(dat00)),
+                "p"=colMeans( dat00, na.rm=T ),
+                "b"=b, "est.b"=est.b, "a"=a, "est.a"=est.a )
     item$thresh <- item$a * item$b
     # add results dependency parameter for item clusters
     item$itemcluster <- itemcluster
@@ -607,17 +606,17 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
 
 #     dat2.li, itemcluster0,
 #                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp
-    datalist <- list( pattern.in.data = pattern.in.data, dat0 = dat0,
-                    dat2 = dat2, dat2.resp = dat2.resp, dat2.li = dat2.li,
-                    dat2.ld = dat2.ld, dat2.li.resp=dat2.li.resp,
-                    dat2.ld.resp = dat2.ld.resp, dp.ld = dp.ld, CC = CC,
-                    bdat2.li = bdat2.li, bdat2.li.resp = bdat2.li.resp,
-                    itemcluster0 = itemcluster0, dat3.ld = dat3.ld
+    datalist <- list( pattern.in.data=pattern.in.data, dat0=dat0,
+                    dat2=dat2, dat2.resp=dat2.resp, dat2.li=dat2.li,
+                    dat2.ld=dat2.ld, dat2.li.resp=dat2.li.resp,
+                    dat2.ld.resp=dat2.ld.resp, dp.ld=dp.ld, CC=CC,
+                    bdat2.li=bdat2.li, bdat2.li.resp=bdat2.li.resp,
+                    itemcluster0=itemcluster0, dat3.ld=dat3.ld
                             )
-    res <- list( "N.itemclusters" = CC, "item" = item, "iter" = iterend, "dev" = dev,
-                    "delta" = delta, "b" = b, "a" = a, "mu" = mu, "sigma" = sigma,
-                    "alpha1"=alpha1, "alpha2"=alpha2, "ic" = ic, "theta.k" = theta.k, "deviance" = dev,
-                    "pattern" = pattern    , "datalist" = datalist                )
+    res <- list( "N.itemclusters"=CC, "item"=item, "iter"=iterend, "dev"=dev,
+                    "delta"=delta, "b"=b, "a"=a, "mu"=mu, "sigma"=sigma,
+                    "alpha1"=alpha1, "alpha2"=alpha2, "ic"=ic, "theta.k"=theta.k, "deviance"=dev,
+                    "pattern"=pattern    , "datalist"=datalist                )
     class(res) <- "rasch.copula"
     return(res)
         }
@@ -636,18 +635,18 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         # use this function for log likelihood calculation
         # look for items which change parameters for necessary update
         G <- GG
-        ind.b <- which( b != rescopula$b )
-        ind.a <- which( a != rescopula$a )
-        ind.delta <- which( delta != rescopula$delta )
-        ind.alpha1 <- ( alpha1 != rescopula$alpha1 )    + ( alpha2 != rescopula$alpha2 )
+        ind.b <- which( b !=rescopula$b )
+        ind.a <- which( a !=rescopula$a )
+        ind.delta <- which( delta !=rescopula$delta )
+        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
         if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
         itemset <- union( ind.b, ind.a )
         itemset <- union( itemset, ind.alpha )
         # update term local independence
         li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
         # update terms item dependence parameters
-        ld.update <- sapply( 1:CC, FUN = function(cc){
-                g1 <- intersect( which( itemcluster == cc ), itemset )
+        ld.update <- sapply( 1:CC, FUN=function(cc){
+                g1 <- intersect( which( itemcluster==cc ), itemset )
                 if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                 v1
                     } )
@@ -656,7 +655,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         ndat2 <- nrow(dat2.ld)
         ntheta <- length(theta.k)
         pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
-        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each = ndat2 ), ]
+        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each=ndat2 ), ]
         # define indices for persons
         ind.dat2 <- rep( seq(1,ndat2), ntheta )
         g1 <- rescopula$g1
@@ -679,16 +678,16 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 m1.cc <- pqjk.theta.k.tt[, dp.ld.cc$independent$items ]
                 v1.cc <- dp.ld.cc$independent$N.Index1
                 # product under independence
-#                Fpjk.cc <- .rowProds.bundle( m1 = m1.cc, v1 = v1.cc)
-                Fpjk.cc <- .rowProds2.bundle( m1 = m1.cc, v1 = v1.cc)
+#                Fpjk.cc <- .rowProds.bundle( m1=m1.cc, v1=v1.cc)
+                Fpjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                 #print( dim(m1.cc))
                 # evaluate likelihood
                 g1.tt <- ( rowSums(Fpjk.cc * dat3.ld.cc ) )^dat2.ld.resp[ind.dat2,cc]
                 # product under dependence
                 m1.cc <-  pqjk.theta.k.tt[, dp.ld.cc$dependent$items ]
                 v1.cc <- dp.ld.cc$dependent$N.Index1
-#                F0pjk.cc <- .rowMins.bundle( m1 = m1.cc, v1 = v1.cc)
-                F0pjk.cc <- .rowMins2.bundle( m1 = m1.cc, v1 = v1.cc)
+#                F0pjk.cc <- .rowMins.bundle( m1=m1.cc, v1=v1.cc)
+                F0pjk.cc <- .rowMins2.bundle( m1=m1.cc, v1=v1.cc)
                 F1pjk.cc <- F0pjk.cc %*% t(dp.ld.cc$calc)
                 g2.tt <- ( rowSums(F1pjk.cc * dat3.ld.cc) )^dat2.ld.resp[ind.dat2,cc]
                 g3.tt <- ( 1 - delta[cc] ) * g1.tt + delta[cc] * g2.tt
@@ -697,14 +696,14 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             }
                 res <- g1
                 g1 <- rowProds2( g1 )
-                ll.theta.k <- matrix( g1, ncol= ntheta )
+                ll.theta.k <- matrix( g1, ncol=ntheta )
                 # casewise: integrate over the product
                 # print(ll.theta.k)
                 # print( outer( M1, wgt.theta ) )
 #                lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
 #                ll1 <- sum( pattern$freqwgt * lli )
                 # one group
-                if (G == 1){
+                if (G==1){
                     # casewise: integrate over the product evaluate at grid of trait density
                     lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
                     ll1 <- sum( lli * pattern$freqwgt)
@@ -720,7 +719,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             }
 
 #                ll1 <- sum( log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) ) )
-                res <- list( "ll" = ll1, "lli" = lli )
+                res <- list( "ll"=ll1, "lli"=lli )
                 return(res)
                 }
 #----------------------------------------------------------------------------------------------
@@ -738,7 +737,7 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
         ndat2 <- nrow(dat2.ld)
         ntheta <- length(theta.k)
         pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
-        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each = ndat2 ), ]
+        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each=ndat2 ), ]
         # define indices for persons
         ind.dat2 <- rep( seq(1,ndat2), ntheta )
         g1 <- matrix( 0, nrow(pjk.theta.k.tt), CC + 1 )
@@ -759,15 +758,15 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                 m1.cc <- pqjk.theta.k.tt[, dp.ld.cc$independent$items ]
                 v1.cc <- dp.ld.cc$independent$N.Index1
                 # product under independence
-#                Fpjk.cc <- .rowProds.bundle( m1 = m1.cc, v1 = v1.cc)
-                Fpjk.cc <- .rowProds2.bundle( m1 = m1.cc, v1 = v1.cc)
+#                Fpjk.cc <- .rowProds.bundle( m1=m1.cc, v1=v1.cc)
+                Fpjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                 # evaluate likelihood
                 g1.tt <- ( rowSums(Fpjk.cc * dat3.ld.cc ) )^dat2.ld.resp[ind.dat2,cc]
                 # product under dependence
                 m1.cc <-  pqjk.theta.k.tt[, dp.ld.cc$dependent$items ]
                 v1.cc <- dp.ld.cc$dependent$N.Index1
-#                F0pjk.cc <- .rowMins.bundle( m1 = m1.cc, v1 = v1.cc)
-                F0pjk.cc <- .rowMins2.bundle( m1 = m1.cc, v1 = v1.cc)
+#                F0pjk.cc <- .rowMins.bundle( m1=m1.cc, v1=v1.cc)
+                F0pjk.cc <- .rowMins2.bundle( m1=m1.cc, v1=v1.cc)
                 F1pjk.cc <- F0pjk.cc %*% t(dp.ld.cc$calc)
                 g2.tt <- ( rowSums(F1pjk.cc * dat3.ld.cc) )^dat2.ld.resp[ind.dat2,cc]
                 g3.tt <- ( 1 - delta[cc] ) * g1.tt + delta[cc] * g2.tt
@@ -776,9 +775,9 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                             }
                 res <- g1
                 g1 <- rowProds2( g1 )
-                ll.theta.k <- matrix( g1, ncol= ntheta )
+                ll.theta.k <- matrix( g1, ncol=ntheta )
                 # one group
-                if (G == 1){
+                if (G==1){
                     # casewise: integrate over the product evaluate at grid of trait density
                     lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
                     ll1 <- sum( lli * pattern$freqwgt)
@@ -792,8 +791,8 @@ rasch.copula <- function( dat, itemcluster, group = NULL,
                         ll1 <- ll1 + sum( lli[[gg]] * pattern[,gg+2] )
                                     }
                             }
-                res <- list( "ll"=ll1, "g1"=res, "b" = b, "a" = a, "delta"=delta,
-                                    "alpha1" = alpha1, "alpha2" = alpha2, "lli" = lli )
+                res <- list( "ll"=ll1, "g1"=res, "b"=b, "a"=a, "delta"=delta,
+                                    "alpha1"=alpha1, "alpha2"=alpha2, "lli"=lli )
                 }
 #----------------------------------------------------------------------------------------------
 
